﻿using Microsoft.AspNetCore.Cors.Infrastructure;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using Microsoft.AspNetCore.Routing;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace WebAPI.Common
{
    public class CorsMiddleware
    {
        private readonly RequestDelegate next;

        public CorsMiddleware(RequestDelegate next)
        {
            this.next = next;
        }
        public async Task Invoke(HttpContext context)
        {
            if (context.Request.Headers.ContainsKey(CorsConstants.Origin))
            {
                context.Response.Headers.Add("Access-Control-Allow-Origin", context.Request.Headers["Origin"]);
                context.Response.Headers.Add("Access-Control-Allow-Methods", "PUT,POST,GET,DELETE,OPTIONS,HEAD,PATCH");
                context.Response.Headers.Add("Access-Control-Allow-Headers", context.Request.Headers["Access-Control-Request-Headers"]);
                context.Response.Headers.Add("Access-Control-Allow-Credentials", "true");

                if (context.Request.Method.Equals("OPTIONS"))
                {
                    context.Response.StatusCode = StatusCodes.Status200OK;
                    return;
                }
            }

            await next(context);
        }
    }




        public class GlobalExceptionFilter : IExceptionFilter
        {
            public void OnException(ExceptionContext context)
            {
                Exception ex = context.Exception;
               
                context.ExceptionHandled = true; //代表异常已经处理，不会再跳转到开发调试时的异常信息页，可以跳转到我们下面自定义的方法中。若开发过程可以将　　　　　　  该行注释掉，则直接抛出异常调试

                //通过HTTP请求头来判断是否为Ajax请求，Ajax请求的request headers里都会有一个key为x-requested-with，值“XMLHttpRequest”
                var requestData = context.HttpContext.Request.Headers.ContainsKey("x-requested-with");
                bool IsAjax = false;
                if (requestData)
                {
                    IsAjax = context.HttpContext.Request.Headers["x-requested-with"] == "XMLHttpRequest" ? true : false;
                }
                if (!IsAjax)//不是异步请求则跳转页面，异步请求则返回json
                {
                    context.RouteData = new Microsoft.AspNetCore.Routing.RouteData();
                    context.RouteData.Values.Add("controller", "home");
                    context.RouteData.Values.Add("action", "error");
                    context.Result = RedirectHelper.UrlFail(context.RouteData);
                }
                else
                    context.Result = RedirectHelper.JsonError();

            }
        }

    public class RedirectHelper
    {

        public static AjaxModel AjaxModel(int code, string msg, dynamic data = null, string url = null)
        {
            return new AjaxModel()
            {
                Code = code,
                Msg = msg,
                Data = data,
                Url = url
            };
        }
        public static Microsoft.AspNetCore.Mvc.IActionResult JsonError()
        {
            JsonResult json = new JsonResult(AjaxModel(500, "系统出错啦"));
            return json;
        }

        public static ActionResult UrlFail(RouteData route)
        {
            RedirectToActionResult result = new RedirectToActionResult(route.Values["action"].ToString(), route.Values["controller"].ToString(), null);
            return result;
        }

    }

    /// <summary>
    /// Json返回结构
    /// </summary>
    public class AjaxModel
    {
        public int Code { get; set; }
        public string Msg { get; set; }
        public object Data { get; set; }
        public string Url { get; set; }
    }
}
